###################################################################
###	y(t | Z1,Z2,Z3; b) = Z1%*%beta1 + b1	
###	LAMBDA_R(t | Z1,Z2,Z3; b)  = H( LR(t)*exp(Z2%*%beta2+b2) )	
###	LAMBDA_T(t | Z1,Z2,Z3; b)  = G( LT(t)*exp(Z3%*%beta3+b1*phi1+b2*phi2) )
###################################################################

###################################################################
#	"H", "G" functions 
###################################################################
  TRANSFORMATION="LOG";   ###otherwise, TRANSFORMATION="Box-Cox"
  rH <- 0
  rG <- 1

Model.Info=paste(TRANSFORMATION," Transformation: ", "(rH,rG)=","(",rH,",",rG,")", sep="")
Model.HG = 1000*rH + 10*rG

  if (TRANSFORMATION=="LOG"){
      if (rH==0){
         H <- function(x) {x}
         Hinv <- function(x) {x}
         H1d <- function(x)  {1}
         H2d <- function(x) {0}
         H3d <- function(x) {0}
      } else if (rH > 0){
         H <- function(x) {log(1+rH*x)/rH}
         Hinv <- function(x) {(exp(rH*x)-1)/rH}
         H1d <- function(x) {1/(1+rH*x)}
         H2d <- function(x) {-rH/(1+rH*x)^2}
         H3d <- function(x) {2*rH*rH/(1+rH*x)^3}
      }
      if (rG==0){
         G <- function(x) {x}
         Ginv <- function(x) {x}
         G1d <- function(x)  {1}
         G2d <- function(x) {0}
         G3d <- function(x) {0}
      } else if (rG > 0){
         G <- function(x) {log(1+rG*x)/rG}
         Ginv <- function(x) {(exp(rG*x)-1)/rG}
         G1d <- function(x) {1/(1+rG*x)}
         G2d <- function(x) {-rG/(1+rG*x)^2}
         G3d <- function(x) {2*rG*rG/(1+rG*x)^3}
      }
  } else{
       if (rH==0){
         H <- function(x) {log(1+x)}
         Hinv <- function(x) {(exp(x)-1)}
         H1d <- function(x) {1/(1+x)}
         H2d <- function(x) {-1(1+x)^2}
         H3d <- function(x) {2/(1+x)^3}
      } else if (rH > 0){
         H <- function(x) {((1+x)^rH-1)/rH}
         Hinv <- function(x) {(rH*x+1)^(1/rH)-1}
         H1d <- function(x) {(1+x)^(rH-1)}
         H2d <- function(x) {(rH-1)*(1+x)^(rH-2)}
         H3d <- function(x) {(rH-1)*(rH-2)*(1+x)^(rH-3)}
      }
      if (rG==0){
         G <- function(x) {log(1+x)}
         Ginv <- function(x) {(exp(x)-1)}
         G1d <- function(x) {1/(1+x)}
         G2d <- function(x) {-1/(1+x)^2}
         G3d <- function(x) {2/(1+x)^3}
      } else if (rG > 0){
         G <- function(x) {((1+x)^rG-1)/rG}
         Ginv <- function(x) {(rG*x+1)^(1/rG)-1}
         G1d <- function(x) {(1+x)^(rG-1)}
         G2d <- function(x) {(rG-1)*(1+x)^(rG-2)}
         G3d <- function(x) {(rG-1)*(rG-2)*(1+x)^(rG-3)}
      }
  }

###################################################################
#	Gauss-Hermite quadrature
###################################################################
library(statmod)
GHQ <- gauss.quad(10, kind="hermite")
NGHQ <- length(GHQ$nodes)
node <- GHQ$nodes
weight <- GHQ$weights

ww <- rbind(rep(weight, rep(NGHQ,NGHQ)),rep(weight,NGHQ))
Nnode <- dim(ww)[2]


###################################################################
#	Datastep func
###################################################################
DataStepE <- function(dataname, IDcol, DeltaRcol){
  dataname <- as.matrix(dataname)
  RE <- NULL;	TE <- NULL;
  PID <- 1  ;
  for (i in 1:(nrow(dataname)-1)){
         ID <- dataname[i,IDcol]
         IDnext <- dataname[(i+1),IDcol]
         if (dataname[i,DeltaRcol] == 1) RE <- rbind(RE, c(dataname[i, ],PID))          
         if (ID != IDnext) {
  		TE <- rbind(TE, c(dataname[i, ], PID))
		PID <- PID+1
         }
  }
  TE <- rbind(TE, c(dataname[nrow(dataname), ], PID))
  if (dataname[nrow(dataname),DeltaRcol] == 1) {
	   RE <- rbind(RE, c(dataname[nrow(dataname), ], PID))
  }
  out <- list(RE=RE, TE=TE);	out;
}

###################################################################
#
#	E-step func
#
###################################################################
Tbb <- function(par_b){	  
  r1 = par_b[1];     r2 = par_b[2];     r3 = par_b[3]; 
  VV <- matrix(c(r1,r3*sqrt(r1*r2),r3*sqrt(r1*r2),r2), 2,2) 
	eV <- eigen(VV)
	squarerootV <- eV$vectors %*% diag(sqrt(eV$values)) %*% t(eV$vectors)
	bb <- sqrt(2)*squarerootV %*% rbind(rep(node, rep(NGHQ,NGHQ)),rep(node,NGHQ))
	bb
}

GetWEIGHT <- function(bb, par_Y, par_R, par_T, par_b, ZZ1, Z2, ZZ2, Z3){
  beta1 =par_Y[1:nbeta1]; sigma2_e =par_Y[nbeta1+1];
  beta2 =par_R[1:nbeta2]; fr =par_R[(nbeta2+1):nR]; 
  beta3 =par_T[1:nbeta3]; phi1 =par_T[nbeta3+1]; phi2 =par_T[nbeta3+2];
            ft =par_T[(nbeta3+3):nT];            
  r1 =par_b[1]; r2 =par_b[2]; r3 =par_b[3];

  ZB3 <- c(Z3%*%beta3);
  ZZB2 <- c(ZZ2%*%beta2);  ZB2 <-c(Z2%*%beta2);
  YZZB1 <- c(YE- ZZ1%*%beta1)
  Fr_Tik <- indR_Tik %*% fr 
  Fr_Xi <- indR_Xi %*% fr   
  Ft_Xi <- indT_Xi %*% ft     
  
  GetGAMMA <- function(b){
  	b1 <- b[1]; b2 <- b[2];	
  	tempY <- -1/(2*sigma2_e)*(b1^2-2*YZZB1*b1)
  	tempR <- b2 + log(H1d(exp(ZZB2+b2)*Fr_Tik))
  	tempY <- indIDY %*% tempY
  	if (length(tempR) == 1) rec <- indIDR %*% rep(tempR, mr)
  	if (length(tempR) > 1)  rec <- indIDR %*% tempR
  	ter <- DELTA*(b1*phi1+b2*phi2+log(G1d(exp(ZB3+b1*phi1+b2*phi2)*Ft_Xi)))
  	HG <- -H(exp(ZB2+b2)*Fr_Xi) -G(exp(ZB3+b1*phi1+b2*phi2)*Ft_Xi)
  	out<- exp(tempY + rec + ter + HG); 	out;
  }
  GAMMA <- apply(bb, 2, GetGAMMA)
  weights <- GAMMA*matrix(ww[1,]*ww[2,],N,Nnode,byrow=TRUE)   
  wwN <- weights/ rowSums(weights);
  wwN[which(is.nan(wwN)=='TRUE')] <- 0;     wwN ;
}



###################################################################
#
#	M-step func
#
###################################################################
 
###################################################################
#	recursive        - c1=f_mr,r  c2=f_mt,t
###################################################################
recursiveR <- function(c1, bb, wwN, wwmr, beta2, Z2, ZZ2){
  ZZB2 <- c(ZZ2%*%beta2);  ZB2 <- c(Z2%*%beta2);
  b2v = matrix(bb[2, ],N,Nnode,byrow=TRUE)  
    
  fr=rep(0,mr);   fr[mr]=c1; 
  dfr_beta2 = matrix(0,mr,nbeta2);    
  dfr_c1 = matrix(0,mr,1);   dfr_c1[mr]=1;   
       
  for (jj in 1:(mr-1)){
    j <- mr-jj
    Fr <- 1- sum(fr[(j+1):mr])
    if (jj==1) dFr_b2 <- -dfr_beta2[mr, ]
    if (jj> 1) dFr_b2 <- colSums(-dfr_beta2[(j+1):mr, ])
    dFr_c1 <- -sum(dfr_c1[(j+1):mr])
        
    exptemp = exp(ZZB2 + b2v[rep(1:N,ni), ])
    H1=H1d(exptemp*Fr);  H2=H2d(exptemp*Fr);  H3=H3d(exptemp*Fr);
    grH2H1 = H2/H1*exptemp;
    grdH2H1= H3/H1*exptemp*exptemp-(H2/H1*exptemp)^2;
    exptemp<-exp(ZB2 + b2v)	
    H1=H1d(exptemp*Fr);  H2=H2d(exptemp*Fr);
    grH1<- matrix(H1*exptemp, nrow=N);	   	  
    grH2<- matrix(H2*exptemp*exptemp, nrow=N);         
    ErH2H1 <- rowSums(grH2H1*wwmr)
    ErdH2H1 <- rowSums(grdH2H1*wwmr)
    ErH1 <- rowSums(grH1*wwN)
    ErH2 <- rowSums(grH2*wwN)

    idxRE <- ifelse(wr[j]<=RE,1,0)*ifelse(wr[j+1]>RE,1,0)
    idxTE <- ifelse(wr[j]<=TE,1,0)*ifelse(wr[j+1]>TE,1,0)
    frtemp1<- sum(idxRE*ErH2H1)
    frtemp2<- sum(idxTE*ErH1)
    frtemp <- 1/fr[j+1] - frtemp1 + frtemp2
    fr[j] <- 1/frtemp
    b2temp1<- idxRE*(ErH2H1*ZZ2 + ErdH2H1*(ZZ2*Fr+matrix(dFr_b2,mr,nbeta2,byrow=TRUE)))
    b2temp2<- idxTE*(ErH1*Z2 + ErH2*(Z2*Fr+matrix(dFr_b2,N,nbeta2,byrow=TRUE)))
    b2temp<- -1/fr[j+1]^2*dfr_beta2[j+1, ] -colSums(b2temp1) +colSums(b2temp2)
    dfr_beta2[j, ]<- -fr[j]^2*b2temp
    c1temp1<- idxRE*ErdH2H1*dFr_c1
    c1temp2<- idxTE*ErH2*dFr_c1
    c1temp <- -1/fr[j+1]^2*dfr_c1[j+1,1] - sum(c1temp1) + sum(c1temp2)
    dfr_c1[j,1] <- -fr[j]^2*c1temp
  }
  cbind(fr,dfr_beta2,dfr_c1)        
}   # end of "recursiveR" #


recursiveT <- function(c2, bb, wwN, wwmr, beta3, phi1, phi2, Z3){
  ZB3 <- c(Z3%*%beta3);
  b1v = matrix(bb[1, ],N,Nnode,byrow=TRUE)
  b2v = matrix(bb[2, ],N,Nnode,byrow=TRUE)
        
  ft=rep(0, mt);   ft[mt]=c2;  
  dft_beta3 = matrix(0,mt,nbeta3); 
  dft_phi1 = matrix(0,mt,1);   dft_phi2 = matrix(0,mt,1);  
  dft_c2 = matrix(0,mt,1);  dft_c2[mt]=1;   
  
  for (jj in 1:(mt-1)){
    j <- mt-jj
    Ft <- 1- sum(ft[(j+1):mt])
    if (jj==1) dFt_b3 <- -dft_beta3[mt, ]
    if (jj> 1) dFt_b3 <- colSums(-dft_beta3[(j+1):mt, ])
    dFt_p1 <- -sum(dft_phi1[(j+1):mt])
    dFt_p2 <- -sum(dft_phi2[(j+1):mt])
    dFt_c2 <- -sum(dft_c2[(j+1):mt]) 
    
    exptemp=exp(ZB3 + b1v*phi1 + b2v*phi2); 
    G1=G1d(exptemp*Ft); G2=G2d(exptemp*Ft); G3=G3d(exptemp*Ft);
    grG2G1 =matrix(G2/G1*exptemp,nrow=N) 
    grdG2G1=matrix(G3/G1*exptemp*exptemp-(G2/G1*exptemp)^2,nrow=N);
    grG1<- matrix(G1*exptemp, nrow=N)	   	  
    grG2<- matrix(G2*exptemp*exptemp, nrow=N);
    ErG2G1 <- rowSums(grG2G1*wwN)
    ErG2G1b1 = rowSums(grG2G1*b1v*wwN)
    ErG2G1b2<- rowSums(grG2G1*b2v*wwN)
    ErdG2G1 <- rowSums(grdG2G1*wwN)
    ErdG2G1b1 <- rowSums(grdG2G1*b1v*wwN)
    ErdG2G1b2<- rowSums(grdG2G1*b2v*wwN)
    ErG1 <- rowSums(grG1*wwN)
    ErG1b1<- rowSums(grG1*b1v*wwN)
    ErG1b2<- rowSums(grG1*b2v*wwN)
    ErG2 <- rowSums(grG2*wwN)
    ErG2b1 <- rowSums(grG2*b1v*wwN)
    ErG2b2 <- rowSums(grG2*b2v*wwN)     		                 
		                 		
    idxTE <- ifelse(wt[j]<=TE, 1, 0)*ifelse(wt[j+1]>TE, 1, 0)
    idxDT <- DELTA*idxTE
    fttemp1<- sum(idxDT*ErG2G1)
    fttemp2<- sum(idxTE*ErG1)
    fttemp <- 1/ft[j+1] - fttemp1 + fttemp2
    ft[j] <- 1/fttemp
    b3temp1<- idxDT*(ErG2G1*Z3 + ErdG2G1*(Z3*Ft+matrix(dFt_b3,N,nbeta3,byrow=TRUE)))
    b3temp2<- idxTE*(ErG1*Z3 + ErG2*(Z3*Ft+matrix(dFt_b3,N,nbeta3,byrow=TRUE)))
    b3temp <- -1/ft[j+1]^2*dft_beta3[j+1, ] + colSums(-b3temp1+b3temp2)
    dft_beta3[j, ] <- -ft[j]^2*b3temp
    p1temp1<- idxDT*(ErG2G1b1 + ErdG2G1b1*Ft + ErdG2G1*dFt_p1)
    p1temp2<- idxTE*(ErG1b1 + ErG2b1*Ft + ErG2*dFt_p1)
    p1temp <- -1/ft[j+1]^2*dft_phi1[j+1,1] - sum(p1temp1) + sum(p1temp2)
    dft_phi1[j,1] <- -ft[j]^2*p1temp
    p2temp1<- idxDT*(ErG2G1b2 + ErdG2G1b2*Ft + ErdG2G1*dFt_p2)
    p2temp2<- idxTE*(ErG1b2 + ErG2b2*Ft + ErG2*dFt_p2)
    p2temp <- -1/ft[j+1]^2*dft_phi2[j+1,1] - sum(p2temp1) + sum(p2temp2)
    dft_phi2[j,1] <- -ft[j]^2*p2temp
    c2temp1<- idxDT*ErdG2G1*dFt_c2
    c2temp2<- idxTE*ErG2*dFt_c2
    c2temp <- -1/ft[j+1]^2*dft_c2[j+1,1] - sum(c2temp1) + sum(c2temp2)
    dft_c2[j,1] <- -ft[j]^2*c2temp
	}
	cbind(ft,dft_beta3,dft_phi1,dft_phi2,dft_c2) 
}   # end of "recursiveT" #  

  
###################################################################
#  M_step
###################################################################
M_step <- function(bb, wwN, wwmr, par_Y, par_R, par_T, par_b, ZZ1, Z2, ZZ2, Z3){ 
    beta1 =par_Y[1:nbeta1];   sigma2_e =par_Y[nbeta1+1];
    beta2 =par_R[1:nbeta2]; 
              fr =par_R[(nbeta2+1):nR];     c1 =fr[mr];
    beta3 =par_T[1:nbeta3];   phi1 =par_T[nbeta3+1];    phi2 =par_T[nbeta3+2];
              ft =par_T[(nbeta3+3):nT];     c2= ft[mt];       
    r1 =par_b[1];   r2 =par_b[2];   r3 =par_b[3];
    
    ZB3 <- c(Z3%*%beta3);
    ZZB2 <- c(ZZ2%*%beta2);  ZB2 <- c(Z2%*%beta2);
    YZZB1 <- c(YE- ZZ1%*%beta1)
    b1v = matrix(bb[1, ],N,Nnode,byrow=TRUE)
    b2v = matrix(bb[2, ],N,Nnode,byrow=TRUE)
    
  #################################################################
  #	ft, ft, dfr, dft, Fr, Ft
  ################################################################# 	
  fdf_R <- recursiveR(c1, bb, wwN, wwmr, beta2, Z2, ZZ2)
    fr = fdf_R[ ,1]; 
    dfr_beta2 = fdf_R[ ,2:(1+nbeta2)];  
    dfr_c1= fdf_R[ ,(2+nbeta2)]; 
    
  fdf_T <- recursiveT(c2, bb, wwN, wwmr, beta3, phi1, phi2, Z3)  
    ft = fdf_T[, 1]; 
    dft_beta3 = fdf_T[ ,2:(1+nbeta3)]; 
    dft_phi1 = fdf_T[ ,(nbeta3+2)]; 
    dft_phi2 = fdf_T[ , (nbeta3+3)]; 
    dft_c2 = fdf_T[ , (nbeta3+4)];
    
  Fr_Tik <- c(indR_Tik %*% fr) 
  Fr_Xi <- c(indR_Xi %*% fr)   
  Ft_Xi <- c(indT_Xi %*% ft)   
  
  if (length(which(fr<0)) > 0){
    c1 <- 1/mr;
    fdf_R <- recursiveR(c1, bb, wwN, wwmr, beta2, Z2, ZZ2)
      fr = fdf_R[ ,1];
      dfr_beta2 = fdf_R[ ,2:(1+nbeta2)];
      dfr_c1= fdf_R[ ,(2+nbeta2)];
  } 
  if (length(which(ft<0)) > 0){
    c2 <- 1/mt;
    fdf_T <- recursiveT(c2, bb, wwN, wwmr, beta3, phi1, phi2, Z3)
    ft = fdf_T[, 1];
      dft_beta3 = fdf_T[ ,2:(1+nbeta3)];
      dft_phi1 = fdf_T[ ,(nbeta3+2)];
      dft_phi2 = fdf_T[ , (nbeta3+3)];
      dft_c2 = fdf_T[ , (nbeta3+4)];
  }            
  
  ###################################################################
  # Hfun1,  Hfun2, Gfun in Mstep
  ###################################################################  
     exptemp = exp(ZZB2 + b2v[rep(1:N,ni), ]) 
     expFr <- exptemp * Fr_Tik
     H1=H1d(expFr);  H2=H2d(expFr);  H3=H3d(expFr);
     gH2H1 =matrix(H2/H1*exptemp,nrow=mr) 
     gdH2H1=matrix(H3/H1*exptemp*exptemp-(H2/H1*exptemp)^2,nrow=mr) 

     exptemp = exp(ZB2 + b2v)    
     expFr <- exptemp * Fr_Xi
     H1=H1d(expFr);   H2=H2d(expFr);   H3=H3d(expFr);
     gH1<- matrix(H1*exptemp, nrow=N)	   	  
     gH2<- matrix(H2*exptemp*exptemp, nrow=N)

     exptemp=exp(ZB3 + b1v*phi1 + b2v*phi2)
     expFt <- exptemp * Ft_Xi
     G1=G1d(expFt);     G2=G2d(expFt);    G3=G3d(expFt);
     gG2G1 =matrix(G2/G1*exptemp,nrow=N) 
     gdG2G1=matrix(G3/G1*exptemp*exptemp-(G2/G1*exptemp)^2,nrow=N)
     gG1<- matrix(G1*exptemp, nrow=N)   	  
     gG2<- matrix(G2*exptemp*exptemp, nrow=N)
  
  #################################################################
  #	Expectations
  #################################################################
  Eb1 <- rowSums(wwN*b1v)
  Eb2 <- rowSums(wwN*b2v) 
  Eb1sq <- rowSums(wwN * b1v^2)
  Eb2sq <- rowSums(wwN * b2v^2)
  Eb1b2 <- rowSums(wwN * b1v * b2v)
 
  EH2H1 <- rowSums(gH2H1*wwmr)
  EdH2H1 <- rowSums(gdH2H1*wwmr)  
  EH1 <- rowSums(gH1*wwN)
  EH2 <- rowSums(gH2*wwN)	 
  EG2G1 <- rowSums(gG2G1*wwN)
  EG2G1b1 <- rowSums(gG2G1*b1v*wwN) 
  EG2G1b2<- rowSums(gG2G1*b2v*wwN)
  EG2G1b1sq=rowSums(gG2G1*b1v^2*wwN) 
  EG2G1b2sq<- rowSums(gG2G1*b2v^2*wwN)
  EG2G1b1b2<- rowSums(gG2G1*b1v*b2v*wwN)  
  EdG2G1 <- rowSums(gdG2G1*wwN)
  EdG2G1b1<- rowSums(gdG2G1*b1v*wwN)  	
  EdG2G1b2<- rowSums(gdG2G1*b2v*wwN)
  EdG2G1b1sq=rowSums(gdG2G1*b1v^2*wwN)
  EdG2G1b2sq<- rowSums(gdG2G1*b2v^2*wwN)
  EdG2G1b1b2<- rowSums(gdG2G1*b1v*b2v*wwN)  
  EG1 <- rowSums(gG1*wwN)
  EG1b1<-  rowSums(gG1*b1v*wwN)  
  EG1b2<- rowSums(gG1*b2v*wwN)
  EG1b1sq=rowSums(gG1*b1v^2*wwN)
  EG1b2sq<- rowSums(gG1*b2v^2*wwN)
  EG1b1b2<-  rowSums(gG1*b1v*b2v*wwN)  
  EG2 <- rowSums(gG2*wwN)
  EG2b1<- rowSums(gG2*b1v*wwN) 
  EG2b2<- rowSums(gG2*b2v*wwN)
  EG2b1sq=rowSums(gG2*b1v^2*wwN)
  EG2b2sq<- rowSums(gG2*b2v^2*wwN)
  EG2b1b2<- rowSums(gG2*b1v*b2v*wwN)
    
  #################################################################
  #	Newton Raphson -  beta2, beta3, phi1, phi2, c1, c2
  ################################################################# 	
  ##### Score functions Recurrent 	
  sbeta2 <- colSums(ni*Z2) + t(ZZ2)%*%(EH2H1*Fr_Tik) - t(Z2)%*%(EH1*Fr_Xi)
  sfjr <- sum(fr)-1
  ##### Score functions Terminal
  sbeta3 <- colSums(DELTA*Z3) + t(Z3)%*%(DELTA*EG2G1*Ft_Xi - EG1*Ft_Xi)	
  sphi1 <- sum(DELTA*Eb1 + DELTA*EG2G1b1*Ft_Xi - EG1b1*Ft_Xi)
  sphi2 <- sum(DELTA*Eb2 + DELTA*EG2G1b2*Ft_Xi - EG1b2*Ft_Xi)
  sfjt <- sum(ft)-1        
  
  ##### derivative of Scores     	
  coefss1<- t(ZZ2*(EH2H1+EdH2H1*Fr_Tik))
  coefs1 <- t(Z2*(EH1+EH2*Fr_Xi))    
  ds1_beta2temp1<- coefss1 %*% (ZZ2*Fr_Tik+indR_Tik%*%dfr_beta2)                   
  ds1_beta2temp2<- coefs1 %*% (Z2*Fr_Xi+indR_Xi%*%dfr_beta2)    
  ds1_beta2 <- ds1_beta2temp1 - ds1_beta2temp2    
  ds1_c1 <- coefss1%*%(indR_Tik%*%dfr_c1) - coefs1%*%(indR_Xi%*%dfr_c1) 
  ds5_beta2 <- t(colSums(dfr_beta2))
  ds5_c1 <- sum(dfr_c1)  
  HHR <- rbind( cbind(ds1_beta2,ds1_c1), 
                cbind(ds5_beta2,ds5_c1)) 
   
  coefss2<- t(Z3*(EG2G1+EdG2G1*Ft_Xi)*DELTA)
  coefs2 <- t(Z3*(EG1+EG2*Ft_Xi))   
  coefss2b1<- t(Z3*(EG2G1b1+EdG2G1b1*Ft_Xi)*DELTA)
  coefs2b1 <- t(Z3*(EG1b1+EG2b1*Ft_Xi))
  coefss2b2<- t(Z3*(EG2G1b2+EdG2G1b2*Ft_Xi)*DELTA)
  coefs2b2 <- t(Z3*(EG1b2+EG2b2*Ft_Xi))  
  ds2_beta3 <- (coefss2 - coefs2)%*%(Z3*Ft_Xi+indT_Xi%*%dft_beta3) 
  ds2_phi1 <- (coefss2b1-coefs2b1)%*%Ft_Xi + (coefss2-coefs2)%*%(indT_Xi%*%dft_phi1)
  ds2_phi2 <- (coefss2b2-coefs2b2)%*%Ft_Xi + (coefss2-coefs2)%*%(indT_Xi%*%dft_phi2) 
  ds2_c2 <- (coefss2 - coefs2)%*%(indT_Xi%*%dft_c2)     
  coefss3<- (EG2G1b1+EdG2G1b1*Ft_Xi)*DELTA
  coefs3 <- EG1b1+EG2b1*Ft_Xi
  coefss3b1sq<- (EG2G1b1sq+EdG2G1b1sq*Ft_Xi)*DELTA
  coefs3b1sq <- EG1b1sq+EG2b1sq*Ft_Xi
  coefss3b1b2<- (EG2G1b1b2+EdG2G1b1b2*Ft_Xi)*DELTA
  coefs3b1b2 <- EG1b1b2+EG2b1b2*Ft_Xi    
  ds3_beta3 <- t(coefss3-coefs3)%*%(Z3*Ft_Xi+indT_Xi%*%dft_beta3)
  ds3_phi1 <- sum((coefss3b1sq-coefs3b1sq)*Ft_Xi+(coefss3-coefs3)*(indT_Xi%*%dft_phi1)) 
  ds3_phi2 <- sum((coefss3b1b2-coefs3b1b2)*Ft_Xi+(coefss3-coefs3)*(indT_Xi%*%dft_phi2))  
  ds3_c2 <- sum((coefss3-coefs3)*(indT_Xi%*%dft_c2))       
  coefss4<- (EG2G1b2+EdG2G1b2*Ft_Xi)*DELTA
  coefs4 <- EG1b2+EG2b2*Ft_Xi
  coefss4b2sq<- (EG2G1b2sq+EdG2G1b2sq*Ft_Xi)*DELTA
  coefs4b2sq <- EG1b2sq+EG2b2sq*Ft_Xi
  coefss4b1b2<- coefss3b1b2
  coefs4b1b2 <- coefs3b1b2   
  ds4_beta3 <- t(coefss4-coefs4)%*%(Z3*Ft_Xi+indT_Xi%*%dft_beta3)
  ds4_phi1 <- sum((coefss4b1b2-coefs4b1b2)*Ft_Xi+(coefss4-coefs4)*(indT_Xi%*%dft_phi1))
  ds4_phi2 <- sum((coefss4b2sq-coefs4b2sq)*Ft_Xi+(coefss4-coefs4)*(indT_Xi%*%dft_phi2))
  ds4_c2 <- sum((coefss4-coefs4)*(indT_Xi%*%dft_c2))
  ds6_beta3 <- t(colSums(dft_beta3))
  ds6_phi1 <- sum(dft_phi1)
  ds6_phi2 <- sum(dft_phi2)
  ds6_c2 <- sum(dft_c2) 
  HHT <- rbind( cbind(ds2_beta3,ds2_phi1,ds2_phi2,ds2_c2),
                cbind(ds3_beta3,ds3_phi1,ds3_phi2,ds3_c2),
                cbind(ds4_beta3,ds4_phi1,ds4_phi2,ds4_c2),
                cbind(ds6_beta3,ds6_phi1,ds6_phi2,ds6_c2))

  #################################################################
  #	Test Determinant
  ################################################################# 
  if(length(which(is.nan(HHR)=='TRUE')) > 0){
		err <- 1
		return(err)
  }
  if(length(which(is.nan(HHT)=='TRUE')) > 0){
		err <- 1
		return(err)
  }
  if(abs(det(HHR)) < 10^(-15) || abs(det(HHT)) < 10^(-15)){
		err <- 1
		return(err)
  }  

  ###################################################################
  #	Update param set 1 - beta1, sigma2_e, r1, r2, r3 
  ###################################################################
  newbeta1 <- solve(t(ZZ1)%*%ZZ1)%*%t(ZZ1)%*%(YE-Eb1[ID.Y])
  newsigma2_e <- sum(YZZB1^2-2*Eb1[ID.Y]*YZZB1+Eb1sq[ID.Y])/sum(ni_Y)
  newr1 <- sum(Eb1sq)/N
  newr2 <- sum(Eb2sq)/N 
  newr3 <- sum(Eb1b2)/N /sqrt(newr1*newr2)    

  #################################################################
  #	Update param set 2 - beta2, beta3, phi1, phi2, c1, c2, fr, ft
  ################################################################# 	 
  paramR <- c(beta2,c1)
  paramT <- c(beta3,phi1,phi2,c2)
  newparamR <- paramR - solve(HHR)%*%c(sbeta2, sfjr)
  newparamT <- paramT - solve(HHT)%*%c(sbeta3,sphi1,sphi2,sfjt)
    newbeta2 <- newparamR[1:nbeta2]; 
    newc1 <- newparamR[nbeta2+1];   
    newbeta3 <- newparamT[1:nbeta3];  
    newphi1 <- newparamT[nbeta3+1]; 
    newphi2 <- newparamT[nbeta3+2]; 
    newc2 <- newparamT[nbeta3+3];   
  newfdf_R <- recursiveR(newc1, bb, wwN, wwmr, newbeta2, Z2, ZZ2)           
  newfdf_T <- recursiveT(newc2, bb, wwN, wwmr, newbeta3, newphi1, newphi2, Z3) 
    newfr = newfdf_R[ ,1]; 
    newft = newfdf_T[, 1];
    
  if (length(which(newfr<0)) > 0){
    newc1 <- 1/mr;     
    newfdf_R <- recursiveR(newc1, bb, wwN, wwmr, newbeta2, Z2, ZZ2)     
    newfr = newfdf_R[ ,1];  
  }  	  
  if (length(which(newft<0)) > 0){
    newc2 <- 1/mt;     
    newfdf_T <- recursiveT(newc2, bb, wwN, wwmr, newbeta3, newphi1, newphi2, Z3)
    newft = newfdf_T[, 1];
  } 
  if(length(which(is.nan(c(newfr,newft))=='TRUE')) > 0){
		err <- 1
		return(err)
  }
  
  newpar_Y <- c(newbeta1, newsigma2_e)
  newpar_R <- c(newbeta2, newfr)
  newpar_T <- c(newbeta3, newphi1, newphi2, newft)
  newpar_b <- c(newr1, newr2, newr3)
  
  c(newpar_Y, newpar_R, newpar_T, newpar_b)   
}   # end of "M_step" #   


	
###################################################################
#             
# covest 
#
###################################################################	
covest <- function(newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3){
  beta1= newpar_Y[1:nbeta1]; sigma2_e= newpar_Y[nbeta1+1];
  beta2 =newpar_R[1:nbeta2]; fr= newpar_R[(nbeta2+1):nR]; 
  beta3= newpar_T[1:nbeta3]; phi1=newpar_T[nbeta3+1]; phi2=newpar_T[nbeta3+2];
            ft= newpar_T[(nbeta3+3):nT];            
  r1= newpar_b[1]; r2= newpar_b[2]; r3= newpar_b[3]; 
 
  ###################################################################
  # Update     in "covest" 
  ###################################################################
  ZB3 <- c(Z3%*%beta3);
  ZZB2 <- c(ZZ2%*%beta2);   ZB2 <- c(Z2%*%beta2);
  YZZB1 <- c(YE- ZZ1%*%beta1)
  Fr_Tik <- c(indR_Tik %*% fr)
  Fr_Xi <- c(indR_Xi %*% fr)
  Ft_Xi <- c(indT_Xi %*% ft)           

  ###################################################################
  #	E-step      in "covest" 
  ###################################################################
  bb <- Tbb(newpar_b)
  wwN <- GetWEIGHT(bb, newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3)
  wwmr <- wwN[rep(1:N, ni), ] 
  b1v = matrix(bb[1, ],N,Nnode,byrow=TRUE)
  b2v = matrix(bb[2, ],N,Nnode,byrow=TRUE)
 
  ###################################################################
  # Hfun1,  Hfun2, Gfun     in "covest"
  ###################################################################   
     exptemp = exp(ZZB2 + b2v[rep(1:N,ni), ]) 
     expFr <- exptemp * Fr_Tik
     H1=H1d(expFr);  H2=H2d(expFr);  H3=H3d(expFr);
     gH2H1 =matrix(H2/H1*exptemp,nrow=mr) 
     gdH2H1=matrix(H3/H1*exptemp*exptemp-(H2/H1*exptemp)^2,nrow=mr) 

     exptemp = exp(ZB2 + b2v)    
     expFr <- exptemp * Fr_Xi
     H1=H1d(expFr);   H2=H2d(expFr);   H3=H3d(expFr);
     gH1<- matrix(H1*exptemp, nrow=N)	   	  
     gH2<- matrix(H2*exptemp*exptemp, nrow=N)

     exptemp=exp(ZB3 + b1v*phi1 + b2v*phi2)
     expFt <- exptemp * Ft_Xi
     G1=G1d(expFt);     G2=G2d(expFt);    G3=G3d(expFt);
     gG2G1 =matrix(G2/G1*exptemp,nrow=N) 
     gdG2G1=matrix(G3/G1*exptemp*exptemp-(G2/G1*exptemp)^2,nrow=N)
     gG1<- matrix(G1*exptemp, nrow=N)   	  
     gG2<- matrix(G2*exptemp*exptemp, nrow=N)
     
  Eb1 <- rowSums(wwN*b1v)
  Eb2 <- rowSums(wwN*b2v) 
  Eb1sq <- rowSums(wwN * b1v^2)
  Eb2sq <- rowSums(wwN * b2v^2)
  Eb1b2 <- rowSums(wwN * b1v * b2v)
 
  EH2H1 <- rowSums(gH2H1*wwmr)
  EdH2H1 <- rowSums(gdH2H1*wwmr)  
  EH1 <- rowSums(gH1*wwN)
  EH2 <- rowSums(gH2*wwN)	 
  EG2G1 <- rowSums(gG2G1*wwN)
  EG2G1b1 <- rowSums(gG2G1*b1v*wwN) 
  EG2G1b2<- rowSums(gG2G1*b2v*wwN)
  EG2G1b1sq=rowSums(gG2G1*b1v^2*wwN) 
  EG2G1b2sq<- rowSums(gG2G1*b2v^2*wwN)
  EG2G1b1b2<- rowSums(gG2G1*b1v*b2v*wwN)  
  EdG2G1 <- rowSums(gdG2G1*wwN)
  EdG2G1b1<- rowSums(gdG2G1*b1v*wwN)  	
  EdG2G1b2<- rowSums(gdG2G1*b2v*wwN)
  EdG2G1b1sq=rowSums(gdG2G1*b1v^2*wwN)
  EdG2G1b2sq<- rowSums(gdG2G1*b2v^2*wwN)
  EdG2G1b1b2<- rowSums(gdG2G1*b1v*b2v*wwN)  
  EG1 <- rowSums(gG1*wwN)
  EG1b1<-  rowSums(gG1*b1v*wwN)  
  EG1b2<- rowSums(gG1*b2v*wwN)
  EG1b1sq=rowSums(gG1*b1v^2*wwN)
  EG1b2sq<- rowSums(gG1*b2v^2*wwN)
  EG1b1b2<-  rowSums(gG1*b1v*b2v*wwN)  
  EG2 <- rowSums(gG2*wwN)
  EG2b1<- rowSums(gG2*b1v*wwN) 
  EG2b2<- rowSums(gG2*b2v*wwN)
  EG2b1sq=rowSums(gG2*b1v^2*wwN)
  EG2b2sq<- rowSums(gG2*b2v^2*wwN)
  EG2b1b2<- rowSums(gG2*b1v*b2v*wwN)

  ED2LY <- matrix(0, nY, nY)    
  ED2LR <- matrix(0, nR, nR)
  ED2LT <- matrix(0, nT, nT)
  ED2Lb <- matrix(0, nb, nb)
  E_DLDL <- matrix(0, npar, npar)
  EDLEDL <- matrix(0, npar, npar)
  
  for (i in 1:N){          
     IDXY <- which(ID.Y==ID.T[i])
     IDXR <- which(ID.R==ID.T[i])	
     Dbeta1 <- matrix(0, nbeta1, Nnode)
     D2beta1 <- matrix(0, nbeta1, nbeta1)
     Dsigma2_e <- matrix(0, 1, Nnode)
     D2sigma2_e <- 0
     D2beta1sigma2_e <-matrix(0, nbeta1, 1)
     tempDbeta2 <- matrix(0, nbeta2, Nnode)
     tempD2beta2 <- matrix(0, nbeta2, nbeta2)
     tempD2beta2fr <- matrix(0, nbeta2, mr)
     tempDfr <- matrix(0, mr, Nnode)
     tempD2fr <- matrix(0, mr, mr)     

     for (k in IDXY){
       ### Y
       Dbeta1 <- Dbeta1 + ZZ1[k, ]%*%t((YZZB1[k]-bb[1,])/sigma2_e)
       Dsigma2_e <- Dsigma2_e + t((YZZB1[k]-bb[1,])^2/(2*sigma2_e^2)-1/(2*sigma2_e))
       D2beta1<- D2beta1 -ZZ1[k, ]%*%t(ZZ1[k, ])/sigma2_e    
       D2beta1sigma2_e<- D2beta1sigma2_e -(YZZB1[k]-Eb1[i])/sigma2_e^2*ZZ1[k, ]        
       D2sigma2_e<- D2sigma2_e + 1/2/sigma2_e^2 - (YZZB1[k]^2-2*YZZB1[k]*Eb1[i]+Eb1sq[i])/sigma2_e^3
     }
 
     for (kk in IDXR){      
       ### Recurrent 
       indrT <- ifelse(wr<=RE[kk],1,0)  
       coef1 <- matrix(EH2H1[kk]*ZZ2[kk,] + EdH2H1[kk]*Fr_Tik[kk]*ZZ2[kk,])        
       tempDbeta2 <- tempDbeta2 + ZZ2[kk,]+ ZZ2[kk,]%*%t(gH2H1[kk, ])*Fr_Tik[kk]
       tempDfr <- tempDfr + ifelse(wr==RE[kk],1,0)/fr + indrT%*%t(gH2H1[kk, ])
       tempD2beta2 <- tempD2beta2 + (coef1*Fr_Tik[kk])%*%t(ZZ2[kk,])
       tempD2beta2fr <- tempD2beta2fr + coef1%*%t(indrT) 
       tempD2fr <- tempD2fr - diag(mr)*ifelse(wr==RE[kk],1,0)/fr^2+indrT%*%t(indrT)*EdH2H1[kk]       
     }


     indrX <- ifelse(wr<=TE[i],1,0)
     indtX <- ifelse(wt<=TE[i],1,0)
     coef2 <- t(DELTA[i]*gG2G1[i, ]-gG1[i, ])
     coef3 <- matrix((EH1[i]+EH2[i]*Fr_Xi[i])*Z2[i, ])    
     coef4 <- DELTA[i]*(EG2G1[i]+EdG2G1[i]*Ft_Xi[i])-(EG1[i]+EG2[i]*Ft_Xi[i])
     coef5 <- DELTA[i]*(EG2G1b1[i]+EdG2G1b1[i]*Ft_Xi[i])-(EG1b1[i]+EG2b1[i]*Ft_Xi[i])
     coef6 <- DELTA[i]*(EG2G1b2[i]+EdG2G1b2[i]*Ft_Xi[i])-(EG1b2[i]+EG2b2[i]*Ft_Xi[i])

     Dbeta2 <- tempDbeta2 - Z2[i, ]%*%t(gH1[i, ])*Fr_Xi[i]
     Dfr<- tempDfr - indrX%*%t(gH1[i, ]) 
     D2beta2 <- tempD2beta2 - (coef3*Fr_Xi[i])%*%t(Z2[i, ]) 
     D2beta2fr <- tempD2beta2fr - coef3%*%t(indrX)       
     D2fr<- tempD2fr - indrX%*%t(indrX)*EH2[i]
    
     Dbeta3 <-  DELTA[i]*Z3[i, ] + Z3[i, ]%*%(coef2*Ft_Xi[i])
     Dft<- DELTA[i]*ifelse(wt==TE[i],1,0)/ft + indtX%*%coef2
     Dphi1 <-  DELTA[i]*t(b1v[i, ]) + (coef2*Ft_Xi[i])*t(b1v[i, ])
     Dphi2 <-  DELTA[i]*t(b2v[i, ]) + (coef2*Ft_Xi[i])*t(b2v[i, ])
     D2beta3<- coef4*Ft_Xi[i]*Z3[i, ]%*%t(Z3[i, ])          
     D2beta3phi1 <- matrix(coef5*Ft_Xi[i]*Z3[i, ])
     D2beta3phi2 <- matrix(coef6*Ft_Xi[i]*Z3[i, ]) 
     D2beta3ft <- coef4*Z3[i, ]%*%t(indtX)
     D2phi1ft <- t(coef5*indtX)
     D2phi2ft <- t(coef6*indtX)    
     D2ft <- -diag(mt)*DELTA[i]*ifelse(wt==TE[i],1,0)/ft^2+indtX%*%t(indtX)*(DELTA[i]*EdG2G1[i]-EG2[i]) 
     D2phi1    <-(DELTA[i]*(EG2G1b1sq[i]+EdG2G1b1sq[i]*Ft_Xi[i])-(EG1b1sq[i]+EG2b1sq[i]*Ft_Xi[i]))*Ft_Xi[i]
     D2phi1phi2<-(DELTA[i]*(EG2G1b1b2[i]+EdG2G1b1b2[i]*Ft_Xi[i])-(EG1b1b2[i]+EG2b1b2[i]*Ft_Xi[i]))*Ft_Xi[i]
     D2phi2    <-(DELTA[i]*(EG2G1b2sq[i]+EdG2G1b2sq[i]*Ft_Xi[i])-(EG1b2sq[i]+EG2b2sq[i]*Ft_Xi[i]))*Ft_Xi[i]  
    
     #random effect
     coefb1 <- t((b1v[i, ]/r1)^2)
     coefb2 <- t((b2v[i, ]/r2)^2)      
     coefb12 <- t(r3*b1v[i, ]*b2v[i, ]/(r1*r2)^0.5)  
     Ecoefb1 <- Eb1sq[i]/r1^2
     Ecoefb2 <- Eb2sq[i]/r2^2
     Ecoefb1b2 <- r3*Eb1b2[i]/(r1*r2)^0.5    
     Dr1 <- -1/2/r1 -1/2/(1-r3^2)*(-coefb1 + coefb12/r1)     
     Dr2 <- -1/2/r2 -1/2/(1-r3^2)*(-coefb2 + coefb12/r2)     
     Dr3 <- (r3-r3/(1-r3^2)*(r1*coefb1-2*coefb12+r2*coefb2)+coefb12/r3)/(1-r3^2)
     D2r1<- 1/2/r1^2 - (2*Ecoefb1/r1-1.5*Ecoefb1b2/r1^2)/(2*(1-r3^2))     
     D2r2<- 1/2/r2^2 - (2*Ecoefb2/r2-1.5*Ecoefb1b2/r2^2)/(2*(1-r3^2)) 
     D2r3<- ((1+r3^2) - (1+3*r3^2)/(1-r3^2)*(Ecoefb1*r1-2*Ecoefb1b2+Ecoefb2*r2) + 4*Ecoefb1b2)/(1-r3^2)^2
     D2r1r2<- Ecoefb1b2/(r1*r2*4*(1-r3^2))
     D2r1r3<- r3/(1-r3^2)^2*(Ecoefb1-Ecoefb1b2/r1)-Ecoefb1b2/r3/(2*(1-r3^2)*r1)
     D2r2r3<- r3/(1-r3^2)^2*(Ecoefb2-Ecoefb1b2/r2)-Ecoefb1b2/r3/(2*(1-r3^2)*r2) 
    
     ED2LY <- ED2LY + rbind(cbind(D2beta1, D2beta1sigma2_e), cbind(t(D2beta1sigma2_e), D2sigma2_e))
     ED2LR <- ED2LR + rbind(cbind(D2beta2, D2beta2fr), cbind(t(D2beta2fr), D2fr))
     ED2LT <- ED2LT + rbind(cbind(D2beta3, D2beta3phi1, D2beta3phi2, D2beta3ft), 
                       	    cbind(t(D2beta3phi1), D2phi1, D2phi1phi2, D2phi1ft),
                       	    cbind(t(D2beta3phi2), D2phi1phi2, D2phi2, D2phi2ft),
                       	    cbind(t(D2beta3ft), t(D2phi1ft), t(D2phi2ft), D2ft))
     ED2Lb <- ED2Lb + cbind(c(D2r1,D2r1r2,D2r1r3), c(D2r1r2,D2r2,D2r2r3), c(D2r1r3,D2r2r3,D2r3))	                 

     DL <- rbind(Dbeta1,Dsigma2_e, Dbeta2,Dfr, Dbeta3,Dphi1,Dphi2,Dft, Dr1,Dr2,Dr3)
     EDL <-  t(DL)*wwN[i,]
     E_DLDL <- E_DLDL + DL%*%(EDL)
     EDL <- colSums(EDL) 
     EDLEDL <- EDLEDL + EDL%*%t(EDL) 
   }  
   ED2L <- rbind(cbind(ED2LY, matrix(0,nY,nR), matrix(0,nY,nT), matrix(0,nY,nb)),
                 cbind(matrix(0,nR,nY), ED2LR, matrix(0,nR,nT), matrix(0,nR,nb)),
                 cbind(matrix(0,nT,nY), matrix(0,nT,nR), ED2LT, matrix(0,nT,nb)),
                 cbind(matrix(0,nb,nY), matrix(0,nb,nR), matrix(0,nb,nT), ED2Lb))
   COV <- -ED2L - (E_DLDL-EDLEDL)  
   out.eigen<-eigen(COV)
   values<-out.eigen$values	
   values <- 1/values
   COV <- (out.eigen$vectors) %*% diag(values) %*% t(out.eigen$vectors);
   COV;   
}


###################################################################
#             
# LogLik 
#
###################################################################	  
GetLogLik <- function(newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3){
    beta1= newpar_Y[1:nbeta1]; sigma2_e= newpar_Y[nbeta1+1];
    beta2 =newpar_R[1:nbeta2]; fr= newpar_R[(nbeta2+1):nR];
    beta3= newpar_T[1:nbeta3]; phi1=newpar_T[nbeta3+1]; phi2=newpar_T[nbeta3+2];
            ft= newpar_T[(nbeta3+3):nT];
    r1= newpar_b[1]; r2= newpar_b[2]; r3= newpar_b[3];
    ###################################################################
    # Update     in "GetLogLik"
    ###################################################################
    ZB2 <- Z2%*%beta2;    ZB3 <- Z3%*%beta3;
    YZZB1 <- YE- ZZ1%*%beta1;
    ZZB2 <- ZZ2%*%beta2;
    Fr_Tik <- indR_Tik %*% fr
    Fr_Xi <- indR_Xi %*% fr
    Ft_Xi <- indT_Xi %*% ft

    ###################################################################
    #	E-step      in "GetLogLik"
    ###################################################################
    bb <- Tbb(newpar_b)
    wwN <- GetWEIGHT(bb, newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3)
    wwmr <- wwN[rep(1:N, ni), ]

    ######## compute f(Oi|b) 
    Getf_Oi <- function(b){
      b1 <- b[1];  b2 <- b[2];
      Ycmp <- indIDY%*%(-log(2*pi*sigma2_e)/2 -(YZZB1-b1)^2/(2*sigma2_e))
      Rtemp <- log(fr[rank(RE)]) + ZZB2+b2 + log(H1d(exp(ZZB2+b2)*Fr_Tik))
      Rcmp <- indIDR%*%Rtemp - H(exp(ZB2+b2)*Fr_Xi)
      logft <- rep(0,length(DELTA))
      for(i in 1:length(DELTA)){
          if(DELTA[i] == 1){
            logft[i]<-ft[which(TE[i] == wt)[1]]
          }
      }
      inexp <- ZB3 + phi1*b1 + phi2*b2
      Ttemp <- logft+DELTA*(inexp+log(G1d(exp(inexp)*Ft_Xi)))
      Tcmp <- Ttemp - G(exp(inexp)*Ft_Xi)
      exp(Ycmp+Rcmp+Tcmp)
    }       
    f_Oi <- apply(bb, 2, Getf_Oi)
    loglik <- sum(log(rowSums(f_Oi*wwN))); loglik;
}
 





                