                                 ###################################################################
###	y(t | Z1,Z2,Z3; b) = Z1%*%beta1 + b1	
###	LAMBDA_R(t | Z1,Z2,Z3; b)  = H( LR(t)*exp(Z2%*%beta2+b2) )	
###	LAMBDA_T(t | Z1,Z2,Z3; b)  = G( LT(t)*exp(Z3%*%beta3+b1*phi1+b2*phi2) )
###################################################################

###################################################################
#	Read data
###################################################################          
load("Ydata.RData")
load("Edata.RData")
source("func.R")

###################################################################
#	Set variables
###################################################################          
out.Edata <- DataStepE(Edata, 1, 3)     
### Longitudinal Data
ID.Y <- Ydata[ ,1]
YE <- Ydata[ ,2]
ZZ1 <- Ydata[,3:5]
### Recurrent Event Data
ID.R <- out.Edata$RE[ ,1]
RE <- out.Edata$RE[ ,2]
ZZ2 <- cbind(1, out.Edata$RE[,5:6])   #add monk_intercept for computation
### Terminal Event Data
ID.T <- out.Edata$TE[ ,1]
TE <- out.Edata$TE[ ,2]
DELTA <- out.Edata$TE[ ,4]
Z3 <- cbind(1, out.Edata$TE[,5:6])    #add monk_intercept for computation
Z2 <- Z3
N <- length(TE) 
 
# ni_Y = vector of #of longitudinal outcomes for each subject
ni_Y <- rep(0, N)  
for(i in 1:N){
		ni_Y[i] <- length(which(ID.Y == ID.T[i]))   
}

# ni = vector of #of recurrent events for each subject
ni <- rep(0, N)
for(i in 1:N){
		ni[i] <- length(which(ID.R == ID.T[i]))    
}    

# ordered recurrent/terminal event times
wr <- sort(RE); 	wt <- sort(TE[which(DELTA==1)])      
# # of Y/recurrent/terminal events
my <- length(YE);		mr <- length(RE); 	mt <- sum(DELTA)

# indIDY = Index matrix of the ith subject's longitudinal outcomes
indIDY <- matrix(0, N, my)
for (i in 1:N){
	indIDY[i, which(ID.Y==ID.T[i])] <- 1
}

# indIDR = Index matrix of the ith subject's recurrent event 
indIDR <- matrix(0, N, mr)
for (i in 1:N){
	indIDR[i, which(ID.R==ID.T[i])] <- 1
}

# I(Tik >= wjr), I(Xi >= wjr), I(Xi >= wjt)
indR_Tik <- ifelse(matrix(RE,mr,mr) >= matrix(wr,mr,mr,byrow=TRUE), 1, 0) 
indR_Xi <- ifelse(matrix(TE,N,mr) >= matrix(wr,N,mr,byrow=TRUE), 1, 0) 
indT_Xi <- ifelse(matrix(TE,N,mt) >= matrix(wt,N,mt, byrow=TRUE), 1, 0) 


##########################################################
####	Set summary points for LR(t), LT(t)
##########################################################
tau=max(TE)
testpt <- tau/c(4, 3, 2);    npt <- length(testpt); 

### I(wr <= testpt), I(wt <= testpt) - summary of simulations
indRtestpt <- ifelse(matrix(wr,npt,mr,byrow=TRUE) <=  matrix(testpt,npt,mr), 1, 0) 
indTtestpt <- ifelse(matrix(wt,npt,mt,byrow=TRUE) <=  matrix(testpt,npt,mt), 1, 0) 

### # of params
nbeta1<- ncol(ZZ1); nbeta2<- ncol(Z2); nbeta3<- ncol(Z3) ;  
nY <- nbeta1+1 ;  nR <- nbeta2+mr ;  nT <- nbeta3+2+mt ;  nb <- 3;
npar <- nY + nR + nT + nb;

 

###################################################################
#
# Main Analysis
#
###################################################################
maxiter = 200;    epsilon= .005 
report <- NULL

###################################################################
#	Initials
###################################################################  
beta10= rep(0, nbeta1) ;  sigma2_e0=1;
beta20= rep(0, nbeta2);  fr0=rep(1/mr,mr);  c1= fr0[mr];   
beta30= rep(0, nbeta3);  phi10=0;  phi20=0;  ft0= rep(1/mt,mt);   c2= ft0[mt];    
r10=.5;   r20=.5;   r30=0;  

par_Y <- c(beta10, sigma2_e0)
par_R <- c(beta20, fr0)
par_T <- c(beta30, phi10, phi20, ft0)
par_b <- c(r10, r20, r30)

err<-0 
n.conv <- NULL
for (iter in 1:maxiter){   
  ###################################################################
  #	E-step 
  ###################################################################
  bb <- Tbb(par_b)
  wwN <- GetWEIGHT(bb, par_Y, par_R, par_T, par_b, ZZ1, Z2, ZZ2, Z3)
  wwmr <- wwN[rep(1:N, ni), ] 
  
  ###################################################################
  #	M-step 
  ###################################################################
  Mstep=M_step(bb, wwN, wwmr, par_Y, par_R, par_T, par_b, ZZ1, Z2, ZZ2, Z3) 
  if(length(Mstep)==1){
	   err <- Mstep; warning("Algorithm failed to converge.");	   
         break;
  }   
 	newpar_Y <- Mstep[1:nY]
 	newpar_R <- Mstep[(nY+1):(nY+nR)]
 	newpar_T <- Mstep[(nY+nR+1):(nY+nR+nT)]
 	newpar_b <- Mstep[(nY+nR+nT+1):npar]  
 	newfr <- newpar_R[(nbeta2+1):nR]
 	newft <- newpar_T[(nbeta3+3):nT]    

  error <- c(newpar_Y-par_Y,newpar_R-par_R,newpar_T-par_T,newpar_b-par_b)    
  n.conv <- rbind(n.conv, c(iter, max(abs(error)) ))  
  if(max(abs(error))< epsilon){break}    
  par_Y = newpar_Y;
  par_R = newpar_R; 
  par_T = newpar_T;                                       
  par_b = newpar_b;
}   # end of 'iter' #
 
loglik <- GetLogLik(newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3)

#######################################################
#
#	Summarize results
#
#######################################################
if (err==0){
  COV <- covest(newpar_Y, newpar_R, newpar_T, newpar_b, ZZ1, Z2, ZZ2, Z3)
    var_Y <- COV[1:nY, 1:nY]
    var_R <- COV[nY+1:nR, nY+1:nR]
    var_T <- COV[(nY+nR)+1:nT, (nY+nR)+1:nT]
    var_b <- COV[(nY+nR+nT+1):npar, (nY+nR+nT+1):npar]
  	
  MR= rbind(cbind(diag(nbeta2-1), matrix(0, nbeta2-1, mr)), 
            cbind(matrix(0, npt, nbeta2-1), indRtestpt)) 
  MT= rbind(cbind(diag(nbeta3+1), matrix(0, nbeta3+1, mt)), 
            cbind(matrix(0, npt, nbeta3+1), indTtestpt)) 
  gR= rbind(cbind(matrix(0,nbeta2-1,1), diag(nbeta2-1), matrix(0,nbeta2-1,mr)),
            cbind(matrix(exp(newpar_R[1])*newfr), matrix(0,mr,nbeta2-1),diag(mr)*exp(newpar_R[1]))) 
  gT= rbind(cbind(matrix(0,nbeta3+1,1), diag(nbeta3+1), matrix(0,nbeta3+1,mt)),
		cbind(matrix(exp(newpar_T[1])*newft), matrix(0,mt,nbeta3+1),diag(mt)*exp(newpar_T[1])))
  estvar_R <- MR %*% (gR %*% var_R %*% t(gR)) %*% t(MR)
  estvar_T <- MT %*% (gT %*% var_T %*% t(gT)) %*% t(MT)
  estvar <- c(diag(var_Y), diag(estvar_R), diag(estvar_T), diag(var_b))

  if (min(estvar) > 0){
     estsd <- sqrt(estvar)
     estpar_R <- c(newpar_R[2:nbeta2], newpar_R[(nbeta2+1):nR]*exp(newpar_R[1]))
     estpar_T <- c(newpar_T[2:(nbeta3+2)], newpar_T[(nbeta3+3):nT]*exp(newpar_T[1]))   
     estpar <- c(newpar_Y, MR%*%estpar_R, MT%*%estpar_T, newpar_b) 	   
	        
     ### Zvalue, Pvalue
     Test <- estpar;     Tsd <- estsd;
     NPidx <- c((nY+nbeta2) : (nY+nbeta2-1+npt), (nY+nbeta2+nbeta3+npt+1):(nY+nbeta2+nbeta3+npt+npt))
     VARidx <- c(nY, (nY+nbeta2+nbeta3+npt+npt+1),(nY+nbeta2+nbeta3+npt+npt+2))
     CORRidx <- (nY+nbeta2+nbeta3+npt+npt+3)       	   
     Test[NPidx] <- log(estpar[NPidx])
     Tsd[NPidx]  <- estsd[NPidx]/estpar[NPidx]        
     Test[CORRidx] <- (log(1+estpar[CORRidx]) - log(1-estpar[CORRidx]))/2
     Tsd[CORRidx]  <- estsd[CORRidx]/((1+estpar[CORRidx])*(1-estpar[CORRidx]))    
     Z_value = Test/Tsd                                       
     P_value = 2*pnorm(abs(Z_value), 0, 1, lower.tail=FALSE)       
     tempreport <- cbind(estpar, estsd, Z_value, P_value)
     report <- rbind(Model.HG, loglik, tempreport)
     rn <- c("Model.HG", "loglik",rep("beta1",nbeta1),"s2_e",
        rep("beta2",nbeta2-1), "LR(tau/4)","LR(tau/2)","LR(tau/3)",
        rep("beta3",nbeta3-1), "phi1", "phi2", 
        "LT(tau/4)","LT(tau/2)","LT(tau/3)", "s2_1", "s2_2", "rho" )      
     rownames(report)=rn; colnames(report)=c("PAR","SE","Z_value","P_value")
     Model.Info
     round(report, digits=4)
  } else {
     warning("The minimum of variance estimates is negative.")
  }  
} #end of if err==0
Model.Info 
 